import logging

from ruxit.api.base_plugin import RemoteBasePlugin

from zabbix_api import Zabbix

logger = logging.getLogger(__name__)

severities = {
    "0": "NOT CLASSIFIED",
    "1": "INFORMATION",
    "2": "WARNING",
    "3": "AVERAGE",
    "4": "HIGH",
    "5": "DISASTER",
}


class ZabbixPlugin(RemoteBasePlugin):
    def query(self, **kwargs) -> None:

        severities_filter = self.config.get("zabbix_severities", "").split(",")
        server_address = self.config.get("zabbix_url", "")
        group = self.topology_builder.create_group(f"Zabbix ({server_address})", f"Zabbix ({server_address})")
        hosts = {}

        zabbix_groups = self.config.get("zabbix_groups", "").split(",")

        z = Zabbix(server_address, self.config["zabbix_username"], self.config["zabbix_password"], logger=logger)
        use_zabbix_server_id = self.config.get("zabbix_use_server_id")
        try:
            for host in z.get_hosts(zabbix_groups):
                groups = ",".join([group["name"] for group in host["groups"]])
                host_id = host["hostid"]
                device_id = f'{host_id}'
                if use_zabbix_server_id:
                    device_id = f'{host_id}_{server_address}'
                device = group.create_device(device_id, f'{host["name"]} ({groups}) ({host_id})')
                device.report_property("Groups", groups)
                device.report_property("Host ID", host_id)
                device.report_property("Zabbix Server", server_address)
                for interface in host["interfaces"]:
                    device.add_endpoint(interface["ip"])

                device.absolute("monitored", 100 * int(host["status"] == "0"))
                hosts[host["hostid"]] = device

            problems = z.get_problems()
            events = z.get_events(event_ids=[problem["eventid"] for problem in problems])

            for event in events:
                if not severities_filter or event["severity"] in severities_filter:
                    for host in event["hosts"]:
                        try:
                            # We get all events, but not all hosts, so the host might not be available at this time
                            device = hosts.get(host["hostid"], None)
                            if device is not None:
                                properties = {
                                    "Event ID": event["eventid"],
                                    "Trigger ID": event["objectid"],
                                    "Event URL": f'{self.config["zabbix_url"].replace("api_jsonrpc.php", "tr_events.php")}?triggerid={event["objectid"]}&eventid={event["eventid"]}',
                                    "dt.davis.analysis_time_budget": "0",
                                    "dt.davis.analysis_trigger_delay": "0",
                                    "dt.event.allow_davis_merge": "false"
                                }
                                description = f'{event["name"]}' "ty"
                                title = f"Zabbix - {severities[event['severity']]} Event {event['eventid']}"
                                device.report_error_event(description, title, properties=properties)
                        except Exception as e:
                            logger.exception(f"Error sending event: {e}")
        except Exception as e:
            logger.exception(f"Error executing Zabbix: {e}")
